IF( ${platform_upper} STREQUAL "ATOM" )
  # the name of the target operating system
  SET(CMAKE_SYSTEM_NAME Linux)
  
  LIST( APPEND PLATFORM_NAMES "LINUX" )  
  LIST( APPEND PLATFORM_NAMES "ATOM" )
  LIST( APPEND PLATFORM_NAMES "LINUX_DRM" )
  LIST( APPEND PLATFORM_NAMES "ALL" )

  EXEC_PROGRAM( uname ARGS -p OUTPUT_VARIABLE PROCESSOR_NAME )
  # which compilers to use for C and C++
  SET( CROSS_COMPILER_PATH "$ENV{TOOLCHAIN_ATOM}/sysroots/${PROCESSOR_NAME}-linux/usr/bin/x86_64-poky-linux/" )  
  SET( GCC_PREFIX x86_64-poky-linux- )
  SET( C_COMPILER ${CROSS_COMPILER_PATH}/${GCC_PREFIX}gcc )
  SET( CXX_COMPILER ${CROSS_COMPILER_PATH}/${GCC_PREFIX}g++ )

  include(CMakeForceCompiler)
  CMAKE_FORCE_C_COMPILER( ${C_COMPILER} GNU)
  CMAKE_FORCE_CXX_COMPILER( ${CXX_COMPILER} GNU)


  # here is the target environment located
  SET(CMAKE_SYSROOT $ENV{TOOLCHAIN_ATOM}/sysroots/congatec-baytrail )
  SET(CMAKE_${CMAKE_SYSROOT} )
  SET(SYSROOT ${CMAKE_SYSROOT})
  SET(TOOLCHAIN $ENV{TOOLCHAIN_ATOM})

  IF(NOT DEFINED ENV{TOOLCHAIN_ATOM}) 
    message(FATAL_ERROR "To compile for ATOM module you have to define an environment variable TOOLCHAIN_ATOM that points to your local yocto_intel/built/tmp directory")
  ENDIF()
    
  # http://lists.openembedded.org/pipermail/openembedded-core/2015-January/100452.html
  # changed the default location of the sources in an unusable way, so using the build directory of the kernel under work is probably the best way
  IF(NOT DEFINED ENV{KERNEL_SRC}) 
    message(FATAL_ERROR "To compile for ATOM module you have to define an environment variable KERNEL_SRC that points to the kernel sources directory of the yocto image (e.g. $ENV{TOOLCHAIN_ATOM}/sysroots/atom_module/usr/src/kernel) ")
  ELSE()
    SET( KERNEL_SRC $ENV{KERNEL_SRC} )
  ENDIF()
    
  LIST(APPEND CMAKE_PROGRAM_PATH ${CROSS_COMPILER_PATH})        

  # adjust the default behaviour of the FIND_XXX() commands:
  # search headers and libraries in the target environment, search
  # programs in the host environment
  set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
  set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
  set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)

  find_program( CROSS_C_COMPILER_FOUND ${C_COMPILER} )
  find_program( CROSS_CXX_COMPILER_FOUND ${CXX_COMPILER} )
  IF(NOT CROSS_C_COMPILER_FOUND OR NOT CROSS_CXX_COMPILER_FOUND )
      MESSAGE(FATAL_ERROR "Unable to find ${C_COMPILER} or ${CROSS_CXX_COMPILER_FOUND} program") 
  ENDIF()
    
  #get_filename_component(CROSS_COMPILER_PATH  ${CROSS_COMPILER_FOUND} DIRECTORY)
  
  SET( ENV{PATH} "$ENV{PATH}:${CROSS_COMPILER_PATH}" )

  include_directories( ${SYSROOT}/usr/include/gstreamer-1.0 ${SYSROOT}/usr/lib/glib-2.0/include ${SYSROOT}/usr/include/glib-2.0 ${SYSROOT}/usr/include/libdrm)

  SET_PROPERTY(GLOBAL PROPERTY RULE_LAUNCH_COMPILE ccache)
  SET_PROPERTY(GLOBAL PROPERTY RULE_LAUNCH_LINK ccache) 

  add_definitions( -DDIRECTFB -DPLATFORM_DRM -DATOM -DMESA_EGL_NO_X11_HEADERS )
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} --sysroot=${CMAKE_SYSROOT}")
  set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} --sysroot=${CMAKE_SYSROOT}")


ENDIF()
