string( TOUPPER "$ENV{PLATFORM}" platform_upper )
#set(CMAKE_DISABLE_IN_SOURCE_BUILD ON)
#set(CMAKE_DISABLE_SOURCE_CHANGES  ON)

if ("${CMAKE_SOURCE_DIR}" STREQUAL "${CMAKE_BINARY_DIR}")
  message(SEND_ERROR "In-source builds are not allowed.")
endif ()

# Remove 'lib' prefix for shared libraries on Windows
if(WIN32)
  set(CMAKE_SHARED_LIBRARY_PREFIX "")
endif(WIN32)

cmake_minimum_required(VERSION 3.0.2)
set(CMAKE_INCLUDE_CURRENT_DIR ON)

MACRO(BUILD_LIBRARY args )    
    SET(_args ${ARGV})
    list (REMOVE_AT _args 0)   
    IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )      
      string( FIND ${ARGV0} "QT" POS )
        if( NOT ${POS} EQUAL -1 )
            set(CMAKE_AUTOUIC ON)
            set(CMAKE_AUTOMOC ON)
            set(CMAKE_AUTORCC ON)
            add_definitions(${QT_DEFINITIONS})
            include_directories( ${QT_INCLUDE_DIRECTORIES} )
        endif()
        add_library( ${_args} )
    ELSE()
        MESSAGE( "Skipping ${ARGV1} for platform ${ARGV0}" )
    ENDIF()
ENDMACRO() 

MACRO(BUILD_EXECUTABLE args )
    SET(_args ${ARGV})
    list (REMOVE_AT _args 0)
    IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )        
        string( FIND ${ARGV0} "QT" POS )
        if( NOT ${POS} EQUAL -1 )
            set(CMAKE_AUTOUIC ON)
            set(CMAKE_AUTOMOC ON)
            set(CMAKE_AUTORCC ON)
            add_definitions(${QT_DEFINITIONS})
            include_directories( ${QT_INCLUDE_DIRECTORIES} )
        endif()
        add_executable( ${_args} )
    ELSE()
        MESSAGE( "Skipping ${ARGV1} for platform ${ARGV0}" )
    ENDIF()
ENDMACRO() 

MACRO(LINK_LIBRARIES args )
    IF( TARGET ${ARGV1} )
        SET(_args ${ARGV})
        list (REMOVE_AT _args 0)
        IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )
            TARGET_LINK_LIBRARIES( ${_args} )
        ENDIF()
    ENDIF()
ENDMACRO() 

MACRO(INCLUDE_DIRS args )
    SET(_args ${ARGV})
    list (REMOVE_AT _args 0)
    list (REMOVE_AT _args 0)
    IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )      	
	    FOREACH( path ${_args} )
		IF( IS_ABSOLUTE ${path} )
		  LIST( APPEND TEMP_${ARGV1} ${path} )
		ELSE()
		  LIST( APPEND TEMP_${ARGV1} ${CMAKE_CURRENT_LIST_DIR}/${path} )
	        ENDIF()
	    ENDFOREACH()
    	INCLUDE_DIRECTORIES( ${_args} )
    ENDIF()
    SET( INCLUDE_DIRECTORIES_${ARGV1} ${TEMP_${ARGV1}} CACHE TYPE INTERNAL )	
    SET( INCLUDE_DIRECTORIES_${ARGV1} ${TEMP_${ARGV1}} PARENT_SCOPE )
ENDMACRO() 


MACRO(DEPENDENCIES args )   
    IF( TARGET ${ARGV1} )
        SET(_args ${ARGV})
        list (REMOVE_AT _args 0)
        IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )
          list (REMOVE_AT _args 0)
          FOREACH( dependency ${_args} )		    
		    #MESSAGE( "ARGS: ${ARGV1} dependency: ${dependency} Direct: ${INCLUDE_DIRECTORIES_${dependency}}" )
                    target_include_directories( ${ARGV1} PUBLIC ${INCLUDE_DIRECTORIES_${dependency}} )
          ENDFOREACH()
          add_dependencies( ${ARGV1} ${_args} )
        ENDIF()
    ENDIF()
ENDMACRO() 

MACRO(DEFINITIONS args )
    SET(_args ${ARGV})
    list (REMOVE_AT _args 0)
    IF( ";${PLATFORM_NAMES};" MATCHES ";${ARGV0};" )      
    	add_definitions( ${_args} )
    ENDIF()
ENDMACRO() 

MACRO(CMAKE_PLATFORMS args return_list)
    FILE(GLOB_RECURSE new_list ${args}/*.cmake)    
    LIST(REMOVE_DUPLICATES new_list)
    LIST(APPEND ${return_list} ${new_list})
ENDMACRO() 

# retrieve all cmake platform files
CMAKE_PLATFORMS( ${CMAKE_CURRENT_LIST_DIR} platform_files )

# init platform specific definitions
FOREACH(platform ${platform_files})  
  # get extension without dot (get_filename_component returns it with dot)
  get_filename_component( PLATFORM_NAME ${platform} NAME_WE )   
  string( TOUPPER "${PLATFORM_NAME}" platform_upper2 )
  LIST( APPEND AVAILABLE_PLATFORMS ${platform_upper2} )
  IF( ${platform_upper} STREQUAL ${platform_upper2} )
    # get path to platform specific cmake files  
    get_filename_component( PLATFORM_DIR ${platform} DIRECTORY ) 
    # include the platform file, after specifiying the platform name for platform specific macros
    SET(PLATFORM ${platform_upper2})
    include( ${platform} )
  ENDIF()  
ENDFOREACH()

IF( NOT DEFINED PLATFORM )  
  MESSAGE( FATAL_ERROR "
  Platform $ENV{PLATFORM} not found
  Available platforms: ${AVAILABLE_PLATFORMS}
  " )
ELSE()
  MESSAGE( "Platform ${PLATFORM}" )
  SET( CMAKE_RUNTIME_OUTPUT_DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/../build/${PLATFORM} )
  SET( CMAKE_LIBRARY_OUTPUT_DIRECTORY ${CMAKE_CURRENT_LIST_DIR}/../build/${PLATFORM} )
ENDIF()

file(WRITE "${CMAKE_BINARY_DIR}/QtCreatorDeployment.txt" "/\n")

macro(add_deployment_file PLATFORM_NAME SRC DEST)
    #IF( TARGET ${ARGV1} )
        string( TOUPPER "${PLATFORM_NAME}" platform_upper2 )
        IF( ";${PLATFORM_NAMES};" MATCHES ";${platform_upper2};" )
          file(RELATIVE_PATH dst_path "/" ${DEST})
          get_filename_component(NEW_VAR ${CMAKE_RUNTIME_OUTPUT_DIRECTORY} REALPATH)
          file(RELATIVE_PATH src_path ${CMAKE_SOURCE_DIR} ${NEW_VAR})
          file(APPEND "${CMAKE_BINARY_DIR}/QtCreatorDeployment.txt" "${src_path}/${SRC}:${dst_path}\n")
        ENDIF()
    #ENDIF()
endmacro()

macro(add_deployment_directory PLATFORM_NAME SRC DEST)
    file(GLOB_RECURSE files RELATIVE "${CMAKE_CURRENT_SOURCE_DIR}" "${SRC}/*")
    foreach(filename ${files})
	get_filename_component(path ${filename} PATH)
	add_deployment_file(${PLATFORM_NAME} "${filename}" "${DEST}/${path}")
    endforeach(filename)
endmacro()

MACRO(xxd args)
    IF( TARGET ${ARGV0} )
		get_filename_component(pure_name ${ARGV1} NAME)
		if( EXISTS "${CMAKE_CURRENT_LIST_DIR}/${ARGV1}" )
			configure_file( ${CMAKE_CURRENT_LIST_DIR}/${ARGV1} ${pure_name} COPYONLY)				
			SET( SRC ${CMAKE_CURRENT_LIST_DIR}/${ARGV1} )
		else()
			SET( SRC ${ARGV1} )
		endif()
		add_custom_target( ${ARGV0}_${ARGV2} ALL  COMMAND xxd -i ${pure_name} > ${ARGV2}  )
		set_source_files_properties(${ARGV2} PROPERTIES GENERATED TRUE HEADER_FILE_ONLY TRUE)
		add_dependencies( ${ARGV0} ${ARGV0}_${ARGV2} )
		if( ${ARGC} EQUAL 4 )
		    add_dependencies( ${ARGV0}_${ARGV2} ${ARGV3} )
		ENDIF()
		include_directories( ${CMAKE_CURRENT_BINARY_DIR} )
    ENDIF()
ENDMACRO()

MACRO(hgid args)
    IF( TARGET ${ARGV0} )                
                add_custom_target( ${ARGV0}_${ARGV1}_HG ALL COMMAND hg id -i > ${ARGV1} )
                set_source_files_properties(${ARGV1} PROPERTIES GENERATED TRUE HEADER_FILE_ONLY TRUE)
                add_dependencies( ${ARGV0} ${ARGV0}_${ARGV1}_HG )
                include_directories( ${CMAKE_CURRENT_BINARY_DIR} )
    ENDIF()
ENDMACRO()


IF( NOT MSVC)
set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -std=gnu99")
set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++11 -fno-exceptions ")
SET(CMAKE_C_FLAGS_DEBUG "-D_DEBUG -DDEBUG -O0 -Wextra -Wall -Wunreachable-code")
SET(CMAKE_CXX_FLAGS_DEBUG "-D_DEBUG -DDEBUG -O0 -Wextra -Wall -Wunreachable-code")
ENDIF() 

# Check for correct compiler
# Using C compiler GNUCXX for c++ compiler
IF(CMAKE_COMPILER_IS_GNUCC)
  MESSAGE(STATUS "=== GCC C COMPILER DETECTED")
  SET(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -ggdb")
  SET(CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} -ggdb")
ENDIF(CMAKE_COMPILER_IS_GNUCC)

SET( CMAKE_CXX_FLAGS_RELEASE "${CMAKE_CXX_FLAGS_RELEASE} -DQT_NO_DEBUG" )
SET( CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -DQT_NO_DEBUG" )
