#define PLATFORM_DIRECTFB
#define EGL_EGLEXT_PROTOTYPES
#define GL_GLEXT_PROTOTYPES
#include <iostream>
#include <string>
#include "OpenGL.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#include <libdrm/drm_fourcc.h>
#include <EGL/eglext.h>
#include <scmem/lib/scmem.h>

#include "HelperFunctions.h"

using namespace std;

#define EGL_TEXTURE
#ifdef EGL_TEXTURE
#define TEXTURE_TARGET GL_TEXTURE_EXTERNAL_OES
#else
#define TEXTURE_TARGET GL_TEXTURE_2D
#endif

extern EGLNativeDisplayType getNativeDisplay();
extern EGLNativeWindowType getNativeWindow( EGLNativeDisplayType display );
extern void releaseNativeWindow( EGLNativeWindowType& window, EGLNativeDisplayType display );
extern void initDRM( EGLNativeWindowType window );
extern void drmPageFlip(EGLNativeWindowType window);
extern void releaseNativeWindow( EGLNativeWindowType& window, EGLNativeDisplayType display );
extern bool initEGL( EGLNativeDisplayType nativeDisplay, EGLNativeWindowType window, TextureFormat& format, int depth, int samples);
extern void create_native_pixmap( TextureFormat pixmapFormat, unsigned long pixmapwidth, unsigned long pixmapHeight, unsigned long long* physicalAddress );
extern void release_native_pixmap( unsigned long long logicAddress);

EGLDisplay          m_eglDisplay = EGL_NO_DISPLAY;
EGLNativeWindowType m_eglWindow  = 0;
EGLContext          m_eglContext = EGL_NO_CONTEXT;
EGLSurface          m_eglSurface = EGL_NO_SURFACE;
EGLint              m_screenWidth, m_screenHeight;
EGLImageKHR         m_egl_image_khr = EGL_NO_IMAGE_KHR;

const char*         m_fragShaderCode = " #ifdef GL_ES                                           \n"\
                                       " uniform sampler2D texture1;                            \n"\
                                       " varying mediump vec2 	texCoord;                       \n"\
                                       "#else                                                   \n"\
                                       " uniform sampler2D texture1;                            \n"\
                                       " varying vec2 	texCoord;                               \n"\
                                       "#endif                                                  \n"\
                                       " void main()                                            \n"\
                                       " {                                                      \n"\
                                       "  gl_FragColor = texture2D( texture1, texCoord );       \n"\
                                       " }                                                      \n";

const char*         m_fragShaderCodeEGL = " #ifdef GL_ES                                           \n"\
                                        "#extension GL_OES_EGL_image_external : require         \n"\
                                       " uniform samplerExternalOES texture1;                   \n"\
                                       " varying mediump vec2 	texCoord;                       \n"\
                                       "#else                                                   \n"\
                                       " uniform sampler2D texture1;                            \n"\
                                       " varying vec2 	texCoord;                               \n"\
                                       "#endif                                                  \n"\
                                       " void main()                                            \n"\
                                       " {                                                      \n"\
                                       "  gl_FragColor = texture2D( texture1, texCoord );       \n"\
                                       " }                                                      \n";

const char*         m_vertexShaderCode = "#ifdef GL_ES                                                                                      \n"\
                                        " attribute mediump vec3 vsPos;                                                                    \n"\
                                        " attribute mediump vec2  uvPos;                                                                   \n"\
                                        " uniform mediump mat4	projModelMat;                                                              \n"\
                                        " uniform mediump vec2	sizeScale;                                                                 \n"\
                                        " varying mediump vec2	texCoord;                                                                  \n"\
                                        "#else                                                                                             \n"\
                                        " attribute vec3 vsPos;                                                                            \n"\
                                        " attribute vec2  uvPos;                                                                           \n"\
                                        " uniform mat4	projModelMat;                                                                      \n"\
                                        " uniform vec2	sizeScale;                                                                         \n"\
                                        " varying vec2	texCoord;                                                                          \n"\
                                        "#endif                                                                                            \n"\
                                        " void main()                                                                                      \n"\
                                        " {                                                                                                \n"\
                                        "  gl_Position = projModelMat * vec4( vsPos.x * sizeScale.x, vsPos.y * sizeScale.y, vsPos.z, 1 );  \n"\
                                        "  texCoord = vec2( uvPos.s, uvPos.t );                                                            \n"\
                                        " }                                                                                                ";



#define TEX_WIDTH 2048
#define TEX_HEIGHT 2048
#define NUM_TEXTURES 3

unsigned int        m_texID[NUM_TEXTURES] = {0};
unsigned int        m_vbo = 0, m_shader;
unsigned int        m_currentTexID = 0;
unsigned long long  m_cmemAddress[NUM_TEXTURES] = {0};
void*               m_userspacePtr[NUM_TEXTURES] = {0};
bool                g_run = true;

void signalHandler(int /*signum*/)
{
    g_run = false;
}

void render()
{    
    static bool initialized = false;

    eglMakeCurrent(m_eglDisplay, m_eglSurface, m_eglSurface, m_eglContext);
    glClearColor( 1.0f, 1.0f, 1.0f, 1.0f );
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    if( !m_vbo )
    {
        glGenBuffers( 1, &m_vbo );
        glBindBuffer( GL_ARRAY_BUFFER, m_vbo );
        float vertices[] = {
            0.0f, 0.0f, 0.0f,
            0.0f, 0.0f,
            1.0f, 0.0f, 0.0f,
            1.0f, 0.0f,
            0.0f, 1.0f, 0.0f,
            0.0f, 1.0f,
            1.0f, 1.0f, 0.0f,
            1.0f, 1.0f
        };
        glBufferData( GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW );
    }

    // Init texture
    if( !initialized )
    {
        glGenTextures( 3, m_texID );
        glActiveTexture( GL_TEXTURE0 );
        for( int i = 0; i < NUM_TEXTURES; ++i )
        {
            glBindTexture( TEXTURE_TARGET, m_texID[i] );
            // set the various parameters for the current OpenGL texture:
            glTexParameteri(TEXTURE_TARGET, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
            glTexParameteri(TEXTURE_TARGET, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
            glTexParameteri( TEXTURE_TARGET, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
            glTexParameteri( TEXTURE_TARGET, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

#ifdef EGL_TEXTURE
                int cmemdb_fd = scmem_dma_fd( m_cmemAddress[i] );
                printf ("Got dma_buf_fd=%d\n", cmemdb_fd);
                EGLint egl_db_attrs [] = {
                    EGL_WIDTH,                      TEX_WIDTH,
                    EGL_HEIGHT,                     TEX_HEIGHT,
                    EGL_LINUX_DRM_FOURCC_EXT,       DRM_FORMAT_ARGB8888,
                    EGL_DMA_BUF_PLANE0_FD_EXT,      cmemdb_fd,
                    EGL_DMA_BUF_PLANE0_OFFSET_EXT,  0,
                    EGL_DMA_BUF_PLANE0_PITCH_EXT,   8192,
                    EGL_NONE,
                };

                m_egl_image_khr = eglCreateImageKHR (m_eglDisplay, EGL_NO_CONTEXT, EGL_LINUX_DMA_BUF_EXT, NULL, egl_db_attrs);
                if (m_egl_image_khr == EGL_NO_IMAGE_KHR) {
                    eglErrorCheck();
                    fprintf (stderr, "eglCreateImageKHR returned EGL_NO_IMAGE_KHR\n");
                }
                else
                {
                    fprintf (stderr, "eglCreateImageKHR returned a valid handle %p\n", m_egl_image_khr);
                    glEGLImageTargetTexture2DOES(GL_TEXTURE_EXTERNAL_OES, m_egl_image_khr);
                }
#else
                glTexImage2D( TEXTURE_TARGET, 0, GL_RGBA, TEX_WIDTH, TEX_HEIGHT, 0,  GL_RGBA, GL_UNSIGNED_BYTE, (void*) m_userspacePtr[i] );
#endif
        }
        initialized = true;
    }

    glBindTexture( TEXTURE_TARGET, m_texID[m_currentTexID] );

    //unsigned long long afterUpload = t.getDuration();
    if( !m_shader )
    {

        GLuint fragShader = glCreateShader(GL_FRAGMENT_SHADER);
#ifdef EGL_TEXTURE
        glShaderSource(fragShader, 1, (const char**)&m_fragShaderCodeEGL, NULL);
#else
        glShaderSource(fragShader, 1, (const char**)&m_fragShaderCode, NULL);
#endif
        // Compile the source code
        glCompileShader(fragShader);
        // Check if compilation succeeded
        GLint shaderCompiled;
        glGetShaderiv(fragShader, GL_COMPILE_STATUS, &shaderCompiled);
        if (!shaderCompiled)
        {
            // An error happened, first retrieve the length of the log message
            int i32InfoLogLength, i32CharsWritten;
            glGetShaderiv(fragShader, GL_INFO_LOG_LENGTH, &i32InfoLogLength);

            // Allocate enough space for the message and retrieve it
            char* pszInfoLog = new char[i32InfoLogLength+1];
            pszInfoLog[i32InfoLogLength] = 0;
            glGetShaderInfoLog(fragShader, i32InfoLogLength, &i32CharsWritten, pszInfoLog);

            glDeleteShader( fragShader );

            // Displays the error
            EMSG( "Error compiling shader:\n%s\n------------------------------\n%s\n------------------------------", pszInfoLog, m_fragShaderCode );
            delete[] pszInfoLog;
        }
        // Create the fragment shader object
        GLuint vertShader = glCreateShader(GL_VERTEX_SHADER);
        // Load the source code into it
        glShaderSource(vertShader, 1, (const char**)&m_vertexShaderCode, NULL);

        // Compile the source code
        glCompileShader(vertShader);
        glGetShaderiv(vertShader, GL_COMPILE_STATUS, &shaderCompiled);
        if (!shaderCompiled)
        {
            // An error happened, first retrieve the length of the log message
            int i32InfoLogLength, i32CharsWritten;
            glGetShaderiv(vertShader, GL_INFO_LOG_LENGTH, &i32InfoLogLength);

            // Allocate enough space for the message and retrieve it
            char* pszInfoLog = new char[i32InfoLogLength+1];
            pszInfoLog[i32InfoLogLength] = 0;
            glGetShaderInfoLog(vertShader, i32InfoLogLength, &i32CharsWritten, pszInfoLog);

            glDeleteShader( vertShader );

            // Displays the error
            EMSG( "Error compiling shader:\n%s\n------------------------------\n%s\n------------------------------", pszInfoLog, m_vertexShaderCode );
            delete[] pszInfoLog;
        }

        // Create a shader program obj on GPU
        m_shader = glCreateProgram();

        // Attach the fragment and vertex shaders to it
        glAttachShader(m_shader, fragShader);
        glAttachShader(m_shader, vertShader);

        // Link the program
        glLinkProgram(m_shader);
        glUseProgram( m_shader );

        int m_uvAttribIndex = glGetAttribLocation( m_shader, "uvPos" );
        int m_vertexAttribIndex = glGetAttribLocation( m_shader, "vsPos" );
        int m_matrixUniformIndex = glGetUniformLocation( m_shader, "projModelMat");
        int m_scaleUniformIndex = glGetUniformLocation( m_shader, "sizeScale");

        union mat
        {
            struct {
                float        _11, _12, _13, _14;
                float        _21, _22, _23, _24;
                float        _31, _32, _33, _34;
                float        _41, _42, _43, _44;
            };
            float x[16];
        } m;

        memset( &m, 0, sizeof(m) );

        m._11 =  2.0f / m_screenWidth;
        m._22 = -2.0f / m_screenHeight;
        m._33 = -2 / (1000 - 0.1f);
        m._41 = -1.0f;
        m._42 =  1.0f;
        m._43 =  0.1f;
        m._44 =  1.0f;

        glUniformMatrix4fv( m_matrixUniformIndex, 1, GL_FALSE, m.x );
        glUniform2f( m_scaleUniformIndex, m_screenWidth, m_screenHeight );

        glEnableVertexAttribArray( m_vertexAttribIndex );
        // Default vbo defined in RenderGL has texture coordinates, we don't need in a vertex shader
        // but we have to define the stride according to it, so we have 5 floats (3 vertices, 2 UV coordinates)
        glVertexAttribPointer( m_vertexAttribIndex, 3, GL_FLOAT, GL_FALSE, sizeof(float) * 5, 0 );
        // now pass the texture coordinates data
        glEnableVertexAttribArray( m_uvAttribIndex );
        // additionally to the stride value we have to specify an offset for the vertex data when addressing the texture coordinates
        glVertexAttribPointer( m_uvAttribIndex, 2, GL_FLOAT, GL_FALSE, sizeof(float) * 5, (void*) (3 * sizeof(GLfloat)));

        glViewport( 0, 0, m_screenWidth, m_screenHeight );

    }

    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );

    eglSwapBuffers(m_eglDisplay, m_eglSurface);
    drmPageFlip(m_eglWindow);

}

void mainLoop()
{
    // For some reason getch() does not work, as the terminal catches all inputs (maybe missconfiguration on yocto image)
    // So we catch events directly from the input device for now
    int keyboardFD = openKeyboard();
    struct input_event event[64];
    bool toggleTex = false;
    while(g_run)
    {
        fd_set set;
        /* Initialize the file descriptor set. */
        FD_ZERO (&set);
        FD_SET (keyboardFD, &set);
        struct timeval tv;
        tv.tv_sec = 0;
        tv.tv_usec = 0;

         ssize_t bytes = 0;
        /* select returns 0 if timeout, 1 if input available, -1 if error. */
        if( select(FD_SETSIZE, &set, NULL, NULL, &tv) > 0 )
        {
            if( (bytes = read(keyboardFD, event, sizeof(event))) < (ssize_t) sizeof(input_event) )
                EMSG( "Error reading keyboard" );
            else for(size_t i = 0; i < bytes / sizeof(struct input_event); i++)
            {
                if (event[i].type == EV_KEY && event[i].value == 0 )
                {
                    switch( event[i].code )
                    {
                    case KEY_ESC:
                        g_run = false;
                        break;
                    case KEY_R:
                        m_currentTexID = (m_currentTexID + 1) % NUM_TEXTURES;
                        break;
                    case KEY_SPACE:
                        toggleTex = !toggleTex;
                        break;
                    }
                }
            }
        }
        if( toggleTex )
            m_currentTexID = (m_currentTexID + 1) % NUM_TEXTURES;
        // Do rendering
        render();
    }
    close(keyboardFD);
}

void draw_line(unsigned int* frame, int x_start, int y_start, int x_end, int y_end, unsigned int color)
{
    for(int x = x_start; x <= x_end; x++)
    {
        for(int y = y_start; y <= y_end; y++)
        {
            frame[y*2048+x] = color;
        }
    }
}

int main( int /*argc*/, char** /*argv */)
{    
    signal(SIGINT, signalHandler );
    signal(SIGQUIT, signalHandler );
    signal(SIGTERM, signalHandler );

    srand(time(0));
    // Create render handles
    EGLNativeDisplayType nativeDisplay = getNativeDisplay();
    EGLNativeWindowType nativeWindow = getNativeWindow(nativeDisplay);

    TextureFormat format = TEXTURE_A8R8G8B8;
    if( !initEGL(nativeDisplay, nativeWindow, format, 0, 0) )
        return false;

    // Make the context current
    if ( !eglMakeCurrent(m_eglDisplay, m_eglSurface, m_eglSurface, m_eglContext) || !eglErrorCheck()  )
    {
        fprintf( stderr, "Error making context the current one\n");
        return false;
    }

    eglSwapBuffers(m_eglDisplay, m_eglSurface);
    initDRM(nativeWindow);

    eglQuerySurface(m_eglDisplay, m_eglSurface, EGL_WIDTH, &m_screenWidth);
    eglQuerySurface(m_eglDisplay, m_eglSurface, EGL_HEIGHT, &m_screenHeight);

    unsigned int color_green   = 0x000000FF;
    unsigned int color_black = 0x00000000;


    for( int i = 0; i < NUM_TEXTURES; ++i )
    {
        create_native_pixmap( TEXTURE_A8R8G8B8, TEX_WIDTH,TEX_HEIGHT, &m_cmemAddress[i] );

        // userspace pointer
        m_userspacePtr[i] = scmem_map( m_cmemAddress[i], 2048*2048*4 );
        if( m_userspacePtr[i] )
        {
            draw_line((unsigned int*) m_userspacePtr[i], 0, 0, 2043, 2043, color_black);
            //
            draw_line((unsigned int*) m_userspacePtr[i], 0, 483, 2043, 483, color_green<<i*8);
            draw_line((unsigned int*) m_userspacePtr[i], 0, 1564, 2043, 1564, color_green<<i*8);
            draw_line((unsigned int*) m_userspacePtr[i], 63, 0, 63, 2043, color_green<<i*8);
            draw_line((unsigned int*) m_userspacePtr[i], 1984, 0, 1984, 2043, color_green<<i*8);
        }
    }

    // Main render loop
    mainLoop();

    // Release rendering handles
    if( m_egl_image_khr != EGL_NO_IMAGE_KHR )
        eglDestroyImageKHR(m_eglDisplay, m_egl_image_khr );
    eglMakeCurrent(EGL_NO_DISPLAY, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    if( m_eglContext != EGL_NO_CONTEXT )
    {
        eglDestroyContext(m_eglDisplay, m_eglContext);
        m_eglContext = EGL_NO_CONTEXT;
    }
    if( m_eglSurface != EGL_NO_SURFACE )
    {
        eglDestroySurface(m_eglDisplay, m_eglSurface);
        m_eglSurface = EGL_NO_SURFACE;
    }
    eglTerminate(m_eglDisplay);
    releaseNativeWindow( nativeWindow, nativeDisplay );
    //eglReleaseThread();

    for( int i = 0; i < NUM_TEXTURES; ++i )
    {
        if( m_userspacePtr[i] )
            scmem_unmap(m_userspacePtr[i], 2048*2048*4);
        release_native_pixmap(m_cmemAddress[i]);
    }

    return 0;
}
